/* DeRender.h : Hlavickovy soubor s procedurami pro derendering - generovani 3D sceny ze 2D fotek
   Copyright(c) 2002,2003 Jan Curn (Jan.Curn@seznam.cz) 
*/


#ifndef _DERENDER_H_INCLUDED_
#define _DERENDER_H_INCLUDED_

#include "Render.h"


// Typ pro ulozeni polohy a ostatnich parametru fotoaparatu (kamery)
typedef struct {
	Point3D     ViewPoint;            // Hlavni bod pohledu, odpovida mistu v objektivu, kde se paprsky svetla sbihaji (obvykle uprostred clony)
	double      ViewPlaneDistance;    // Vzdalenost bodu pohledu od pohledove (promitaci) roviny, castecne odpovida vzdalenosti ViewPoint od filmu (je to vlastne nastaveni zoomu vzhledem k rozliseni fotek)
	double      ResXDiv2, ResYDiv2;   // Obrazove rozliseni fotek vydelene dvemi
	Point2D     Resolution;           // Rozliseni fotek
	Triangle3D  BasicSynchroTriangle; // Poloha synchronizacniho trojuhelniku kdyz byla kamera v zakladni poloze (tj. ViewPoint=[0,0,0] apod.)

	Point3D		Pixel00_Point;		// Bod v prostoru odpovidajici pixelu [0,0] z fotky
	Vector3D	XAxis_Vector;		// Vektor v prostoru odpovidajici ose X na fotce, musi byt normovany (tj. GetNorm(XAxis_Vector) == 1 )
	Vector3D	YAxis_Vector;		// Vektor v prostoru odpovidajici ose X na fotce, musi byt normovany (tj. GetNorm(XAxis_Vector) == 1 )	
} Camera, *pCamera;



// Typ pro ulozeni parametru sceny a poloh kamer
typedef struct {
	Triangle3D  SynchroTriangle; // Synchronizacni trojuhelnik - potreba pro urceni vzajemne polohy kamer (A...SynchroPoint, B...MCHCP, C...SCHCP)
	Camera      MasterCamera;    // Hlavni kamera
	Camera      SlaveCamera;     // Vedlejsi kamera				 
} DeRenderInfo, *pDeRenderInfo;


// Typ s informacemi od uzivatele potrebnymi pro synchronizaci kamer
typedef struct {
	Point2D_Ex<double>
	         MasterCameraMainControlPoint, // Souradnice (na fotce) hlavni a vedlejsi kamery, kde jsou dva krajni body synchronizacniho trojuhelniku videt v jedne primce - viz synchronizace fotoaparatu 
	         SlaveCameraMainControlPoint, 
	         MasterCameraHelpControlPoint, // Souradnice (na fotce) hlavni a vedlejsi kamery, kde je zbyvajici bod synchronizacniho trojuhelniku (tento neni videt v primce s jinymi body tohoto trojuhelniku)
	         SlaveCameraHelpControlPoint; 
	double   SynchroPointToMCHCPDistance,  // Realne prostorove rozmery synchronizacniho trojuhelniku (SynchroPoint = bod synch. trojuhelniku, ktery je na obou kamerach vzdy v primce s nejakym jinym, "A";  MCHCP=Master Camera Help Control Point, "B";  SCHCP = Slave Camera Help Control Point, "C")
	         SynchroPointToSCHCPDistance,
	         MCHCPToSCHCPDistance; 
} SynchronizationInfo, *pSynchronizationInfo;


// Typ pro ulozeni informaci pro kalibraci zoomu kamery (vlastne zjisteni vzdalenosti promitaci roviny (filmu) k bodu 
// pohledu (bod v kamere, kde se krizi paprsky svetla-obvykle ve clone), s prihlednutim k rozliseni fotek) - viz. kalibrace fotoaparatu
typedef struct {
	Point2D  PixelA, PixelB;       // Poloha obrazu bodu A a B na fotkach z kamery
	double   AToBDistance;         // Realna prostorova vzdalenost bodu A a B
	double   ViewPointToADistance; // Vzdalenost bodu pohledu (typicky prostredek clony) od bodu A
	double   ViewPointToBDistance; // Vzdalenost bodu pohledu (typicky prostredek clony) od bodu B
	Point2D  Resolution;           // Rozliseni fotky kamery, kterou kalibrujeme
} CameraCalibrationInfo, *pCameraCalibrationInfo;


// Typ pro ulozeni polohy obrazu stejneho bodu na fotce z hlavni a vedlejsi kamery
typedef struct {
	Point2D       MasterCameraPixel, // Souradnice ekvivalentnich pixelu 
	              SlaveCameraPixel;
	unsigned int  BillboardSize;     // Velikost obrazu pixelu 
	Color32       Color;             // Barva ve formatu 0x00rrggbb 
} EqualPoint, *pEqualPoint;


// Typ pro ulozeni polohy obrazu stejneho trojuhelniku na fotce z hlavni a vedlejsi kamery
typedef struct {
	unsigned int  PointAIndex,   // Indexy bodu tvoricich trojuhelnik (indexy do pole Scene::EqualPoints)
	              PointBIndex,
	              PointCIndex; 
	unsigned int  TextureSource; // 0=zadna textura, pouze ohraniceni; 1=master, 2=slave kamera
} EqualTriangle, *pEqualTriangle;


// Zakladni struktura s veskerymi informacemi pro DeRendering
struct DeRenderScene { 
	CameraCalibrationInfo  MasterCalibInfo,     // Kalibracni informace o kamere
	                       SlaveCalibInfo;
	SynchronizationInfo    SynchroInfo;         // Synchronizacni informace

	unsigned int           NumOfEqualPoints,    // Pocty prvku v jednotlivych polich                 
	                       NumOfEqualTriangles; 
	pEqualPoint            EqualPoints;         // Pixely na fotkach z hlavni a vedlejsi kamery, ktere jsou obrazem tehoz bodu v prostoru
	pEqualTriangle         EqualTriangles;      // Ekvivalentni trojuhelniky na fotkach z hlavni a vedlejsi kamery				 				 				 

	pColor32               MasterBitmap,        // Pointer na fotky z master a slave kamer, pokud obsahuje 
	                       SlaveBitmap;         // NULL, prislusna fotka neni pritomna				                                    
	DeRenderScene() 
	{
		EqualPoints = NULL;
		EqualTriangles = NULL;					
		MasterBitmap = NULL;
		SlaveBitmap = NULL;
	}
};
typedef DeRenderScene *pDerenderScene;

bool GeneticallyCalibrateDeRenderScene(const DeRenderScene &DRScene, unsigned int GenerationsCount, DeRenderInfo *pSolution );

bool CreateDeRenderInfo(DeRenderInfo *pSol, 
						const SynchronizationInfo &SynchroInfo, 					
						const CameraCalibrationInfo &MasterCalibrationInfo,
						const CameraCalibrationInfo &SlaveCalibrationInfo,
						double MasterViewPlaneDistance = 0,
						double SlaveViewPlaneDistance = 0);

bool OpenDeRenderScene(const char *Filename, DeRenderScene *pScene);
bool SolveDeRenderScene(const DeRenderScene &DRScene, RenderScene *pRScene, const DeRenderInfo *pDRInfo=NULL);
void CloseDeRenderScene(DeRenderScene *pScene);





#endif // #ifndef _DERENDER_H_INCLUDED_
