/* Render.h : Hlavickovy soubor s procedurami pro rendering - zobrazovani 3D objektu
   Copyright(c) 2002,2003 Jan Curn (Jan.Curn@seznam.cz) 
*/


#ifndef _RENDER_H_INCLUDED_
#define _RENDER_H_INCLUDED_


#include "MyProcs.h"
#include "memory.h"


// Struktura pro ulozeni uzivatelskych parametru pohledu (pro rendering) na scenu
typedef struct {
	Point3D    ViewPoint;         // Bod pohledu
	double     HorViewAngle,      // Horizontalni a vertikalni uhel pohledu
	           VerViewAngle;
	short int  ResX, ResY;        // Rozliseni obrazovky, na kterou vykreslujeme
	double     ViewPlaneDistance; // Vzdalenost pohledove (promitaci) roviny od bodu pohledu = "zoom"
	bool       DrawPoints;        // Jestli se maji vykreslovat body

	// z techto polozek (DrawTrianBorders, DrawTrianTextures, DontDrawTriangles)
	// je vzdy prave jedna true, ostatni musi byt false
	bool       DrawTrianBorders;  // Jestli se maji vykreslovat okraje trojuhelniku
	bool       DrawTrianTextures; // Jestli se maji vykreslovat textury na trojuhelnicich (pokud jsou pritomny)
	bool       DontDrawTriangles; // Jestli se trojuhelniky nebudou vubec kreslit
	Color32    BgColor;           // Barva pozadi                
} ViewParams, *pViewParams;


// Typ pro ulozeni vnitrnich informaci pro rendering
typedef struct {
	Point3D     ViewPoint;          // Bod pohledu
	PlaneCoeff  ViewPlane;          // Pohledova (promitaci) rovina
	double      ResXDiv2, ResYDiv2; // Rozliseni vykreslovaci obrazovky vydelene dvemi
	Matrix4x4   Transformation;     // Matice pro transformaci (zakladni -> normalni poloha)
	Matrix4x4   InvTransformation;  // Matice pro inverzni transformaci (normalni -> zakladni poloha)
	double      ViewPlaneDistance;  // Vzdalenost pohledove roviny = zoom
} RenderInfo, *pRenderInfo;



// Makra pro hodnoty PixelInfoElem::Type
#define PIXELINFO_NOOBJECT			(0)
#define PIXELINFO_TRIANGLE			(1)
#define PIXELINFO_POINT				(2)
#define PIXELINFO_TRIANGLEBORDER	(3)
#define PIXELINFO_LINE				(4)

typedef struct {
	unsigned long   Index;      // Index objektu
	unsigned char   Type;       // Typ objektu na prislusnem pixelu, 0=Zadny, 1=Trojuhelnik, 2=Bod, 3=Hranice trojuhelnika (viz. predchozi makra)
	unsigned short  Reserved1;
	unsigned char   Reserved2;  // Pro zarovnani velikosti objektu na 8 bytu 
} PixelInfoElem, *pPixelInfoElem;



// Typ pro ulozeni vyrenderovaneho obrazu
struct RenderedPicture { 
	Point2D         Resolution; // Rozliseni
	pColor32        Pixels;     // Pole s pixely obrazu (zacatek v levem dolnim rohu, pokracuje se po radcich doprava)
	pPixelInfoElem  PixelInfo;  // Informace o objektu na danem pixelu          
	RenderedPicture()
	{		  
		Pixels = NULL;
		PixelInfo = NULL;
		Resolution.X = Resolution.Y = 0;              
	}
};
typedef RenderedPicture *pRenderedPicture;

// Nasledujici objekty se ukladaji do souboru, je tedy nutne zajistit jejich
// stalou velikost
#pragma pack(push, 8)

// Hlavicka RSF (Render Scene File) souboru
typedef struct {
	char           InfoString[3];        // Retezec obsahujici "RSF" (bez NULL na konci)                 
	Point2D        MasterTextureRes;     // Rozliseni textury (fotky) z master kamery
	Point2D        SlaveTextureRes;      // Rozliseni textury (fotky) ze slave kamery
	unsigned long  NumOfPoints;          // Pocet bodu
	unsigned long  NumOfTriangles;       // Pocet trojuhelniku
	unsigned long  NumOfLines;           // Pocet usecek
	Point3D        MasterCameraPosition; // Puvodni pozice master kamery
	Point3D        SlaveCameraPosition;  // Puvodni pozice slave kamery
//	unsigned char  Reserved1; 
//	unsigned long  Reserved2,
//	               Reserved3,
//	               Reserved4;            // Pro zarovnani objektu na 16ti bytovou hranici
} RSFHeader, *pRSFHeader;

// Typ pro ulozeni bodu v RSF souboru
typedef struct {
	Point3D         Position;       // Poloha 
	Color32         Color;          // Barva ve tvaru 0x00rrggbb
	double          Depth;          // Hloubka bodu po renderingu 
	unsigned char   BillboardSize;  // Velikost obrazu bodu                 
 
	//unsigned short  Reserved1;
	//unsigned char   Reserved2;      // Pro zarovnani objektu na 16ti bytovou hranici
} RSFPoint, *pRSFPoint;

// Makra pro hodnoty RSFTriangle::TextureSource
#define TEXTURE_NONE		(0)		// Zadna textura
#define TEXTURE_MASTERCAM	(1)		// Textura z master kamery
#define TEXTURE_SLAVECAM	(2)		// Textura ze slave kamery

// Typ pro ulozeni trojuhelniku v RSF souboru
typedef struct {
	unsigned long   PointAIndex,      // Indexy bodu tvoricich trojuhelnik
	                PointBIndex,
	                PointCIndex; 
	Triangle2D      TexturePosition;  // Souradnice trojuhelniku na obrazku odkud bereme texturu

	Triangle2D      RenderedPosition; // Renderovana poloha na obrazovce
	PlaneCoeff      BasicStatePlane;  // Rovina trojuhelniku v zakladni poloze (tedy po inverzni transformaci - viz RenderPoint(), CreateRenderInfo() apod.)
	unsigned short  TextureSource;    // Zdroj textury, viz vise makra TEXTURE_...

	//unsigned long   Reserved1, Reserved2;
	//unsigned short  Reserved3;        // Pro zarovnani objektu na 16ti bytovou hranici
} RSFTriangle, *pRSFTriangle;


// Typ pro ulozeni usecky v RSF souboru
typedef struct {
	unsigned long   PointsIndexes[2]; // Poloha 
	Color32         Color;            // Barva ve tvaru 0x00rrggbb
} RSFLine, *pRSFLine;

#pragma pack(pop)

// Typ pro ulozeni sceny pro rendering
struct RenderScene {               
	Point2D        MasterTextureRes;
	Point2D        SlaveTextureRes;                 
	Color32        *MasterTexture; 
	Color32        *SlaveTexture;
				 
	unsigned long  NumOfTriangles;
	unsigned long  NumOfPoints;	
	unsigned long  NumOfLines;
	RSFTriangle    *Triangles;		
	RSFPoint       *Points;	
	RSFLine        *Lines;
	
	Point3D        MasterCameraPosition; 
	Point3D        SlaveCameraPosition; 
              
	RenderScene()
	{
		memset( this, 0, sizeof(*this) );
	}	
};
typedef RenderScene *pRenderScene;

// Deklarace vnejsich funkci z 3DProcs.cpp
bool CreateRenderedPictureEx(RenderedPicture *pPicture, const ViewParams &Params, bool AllocPixelInfo);
bool CreateRenderedPicture(RenderedPicture *pPicture, Point2D Resolution);
inline bool CreateRenderedPicture(RenderedPicture *pPicture, const ViewParams &Params)
	{ return( CreateRenderedPictureEx(pPicture, Params, true) ); }
void DestroyRenderedPicture(RenderedPicture *pPicture);

bool OpenRenderScene(const char *Filename, RenderScene *pScene);
bool SaveRenderScene(const char *Filename, const RenderScene &Scene);
bool RenderWholeScene(const RenderScene &Scene, const ViewParams &Params, RenderedPicture *pPicture);
void CloseRenderScene(RenderScene *pScene);

bool RotateViewAroundPoint(ViewParams *pParams, const Point3D &Point, double HorAngleInc, double VerAngleInc);

bool DrawRectToPicture(RenderedPicture *pPicture, Point2D UpperLeft, Point2D BottomRight, Color32 Color);
bool ApplyZoomToPicture( RenderedPicture *pPicture, float Zoom );
bool SetPointRectToPixelInfo(RenderedPicture &Picture, Point2D UpperLeft, Point2D BottomRight, unsigned int PointIndex);
void DrawPictureToPicture(RenderedPicture *pPicture, Point2D Position, const RenderedPicture &Source);
void DrawPictureToPicture(RenderedPicture *pPicture, Point2D Position, const RenderedPicture &Source, float Zoom);
void DrawLineToPicture(RenderedPicture *pPicture, const Line2D &Line, Color32 Color);

bool WriteBitmap(const char *Filename, const RenderedPicture &Picture);



// Funkce a makra pro ekvalizaci histogramu
#define HISTOGRAM_MODE_B			0x00000000 // podle modre barevne slozky
#define HISTOGRAM_MODE_G			0x00000001 // podle modre barevne slozky
#define HISTOGRAM_MODE_R			0x00000002 // podle modre barevne slozky
#define HISTOGRAM_MODE_BRIGHTNESS	0x00000003 // podle modre barevne slozky
#define HISTOGRAM_MODE_COLORS		0x00000004 // podle modre barevne slozky
#define HISTOGRAM_MODE_ADD			0x00000010
#define HISTOGRAM_MODE_RGB			0x00000010

void GetHistogram(const RenderedPicture &Picture, unsigned int *pHistogram, unsigned int Mode);
void EqualizeHistogram(const unsigned int *Histogram, unsigned char *LUT);
void ApplyLUTToPicture(RenderedPicture *pPicture, const unsigned char *LUT, unsigned int Mode);
void EqualizePicture(RenderedPicture *pPicture, unsigned int Mode);
void EqualizeTwoPictures(RenderedPicture *pPictureA, RenderedPicture *pPictureB, unsigned int Mode);

void ApplyBarlettFilter(RenderedPicture *pPicture, unsigned int Size);








#endif // #ifndef _RENDER_H_INCLUDED_



